@@ -19,6 +19,7 @@ import traceback
 from idds.common import exceptions
 from idds.common.constants import (WorkStatus, ProcessingStatus,
                                    CollectionStatus, CollectionType)
+from idds.common.constants import get_work_status_from_transform_processing_status
 from idds.common.utils import setup_logging
 from idds.common.utils import str_to_date
 # from idds.common.utils import json_dumps
@@ -63,6 +64,11 @@ class Collection(Base):
         self.status = CollectionStatus.New
         self.substatus = CollectionStatus.New
 
+        self.total_files = 0
+        self.processed_files = 0
+        self.processing_files = 0
+        self.bytes = 0
+
     @property
     def internal_id(self):
         return self.get_metadata_item('internal_id')
@@ -134,6 +140,11 @@ class Collection(Base):
             self.status = self._collection['status']
             self.substatus = self._collection['substatus']
 
+            self.total_files = self._collection['total_files']
+            self.processed_files = self._collection['processed_files']
+            self.processing_files = self._collection['processing_files']
+            self.bytes = self._collection['bytes']
+
     def to_origin_dict(self):
         return {'scope': self.scope, 'name': self.name}
 
@@ -369,6 +380,14 @@ class Processing(Base):
     def external_id(self, value):
         self.add_metadata_item('external_id', value)
 
+    @property
+    def old_external_id(self):
+        return self.get_metadata_item('old_external_id', [])
+
+    @old_external_id.setter
+    def old_external_id(self, value):
+        self.add_metadata_item('old_external_id', value)
+
     @property
     def task_name(self):
         return self.get_metadata_item('task_name', None)
@@ -559,6 +578,8 @@ class Work(Base):
         self.or_custom_conditions = {}
         self.and_custom_conditions = {}
 
+        self.sliced_global_parameters = None
+
         """
         self._running_data_names = []
         for name in ['internal_id', 'template_work_id', 'initialized', 'sequence_id', 'parameters', 'work_id', 'transforming', 'workdir',
@@ -572,6 +593,11 @@ class Work(Base):
             self._running_data_names.append(name)
         """
 
+    def get_logger(self):
+        if self.logger is None:
+            self.logger = self.setup_logger()
+        return self.logger
+
     def get_class_name(self):
         return self.__class__.__name__
 
@@ -663,6 +689,14 @@ class Work(Base):
     def work_id(self, value):
         self.add_metadata_item('work_id', value)
 
+    @property
+    def parent_workload_id(self):
+        return self.get_metadata_item('parent_workload_id', None)
+
+    @parent_workload_id.setter
+    def parent_workload_id(self, value):
+        self.add_metadata_item('parent_workload_id', value)
+
     @property
     def transforming(self):
         return self.get_metadata_item('transforming', False)
@@ -671,6 +705,14 @@ class Work(Base):
     def transforming(self, value):
         self.add_metadata_item('transforming', value)
 
+    @property
+    def submitted(self):
+        return self.get_metadata_item('submitted', False)
+
+    @submitted.setter
+    def submitted(self, value):
+        self.add_metadata_item('submitted', value)
+
     @property
     def workdir(self):
         return self.get_metadata_item('workdir', None)
@@ -950,7 +992,7 @@ class Work(Base):
     @num_run.setter
     def num_run(self, value):
         self.add_metadata_item('num_run', value)
-        if value is not None and value > 1:
+        if value is not None:
             # for k in self._collections:
             for coll in self.output_collections:
                 if type(coll) in [Collection]:
@@ -1036,10 +1078,28 @@ class Work(Base):
     def get_is_template(self):
         self.is_template
 
-    def sync_global_parameters(self, global_parameters):
+    def sync_global_parameters(self, global_parameters, sliced_global_parameters=None):
+        if sliced_global_parameters:
+            self.sliced_global_parameters = sliced_global_parameters
+
         if global_parameters:
             for key in global_parameters:
-                setattr(self, key, global_parameters[key])
+                sliced_index = None
+                sliced_name = None
+                if self.sliced_global_parameters and key in self.sliced_global_parameters:
+                    sliced_index = self.sliced_global_parameters[key]['index']
+                    sliced_name = self.sliced_global_parameters[key]['name']
+                    if type(global_parameters[key]) in [list, tuple] and sliced_index < len(global_parameters[key]):
+                        pass
+                    else:
+                        sliced_index = None
+                if not sliced_name:
+                    sliced_name = key
+
+                if sliced_index is None:
+                    setattr(self, sliced_name, global_parameters[key])
+                else:
+                    setattr(self, sliced_name, global_parameters[key][sliced_index])
 
     def get_global_parameter_from_output_data(self, key):
         self.logger.debug("get_global_parameter_from_output_data, key: %s, output_data: %s" % (key, str(self.output_data)))
@@ -1093,6 +1153,9 @@ class Work(Base):
             return False
 
     def get_custom_condition_status_value(self, key):
+        if self.output_data and key in self.output_data:
+            return self.output_data[key]
+
         user_key = "user_" + key
         if hasattr(self, user_key):
             key = user_key
@@ -1135,6 +1198,7 @@ class Work(Base):
         Setup logger
         """
         self.logger = logging.getLogger(self.get_class_name())
+        return self.logger
 
     def add_errors(self, error):
         self.errors.append(error)
@@ -1293,14 +1357,14 @@ class Work(Base):
         return to_release_inputs
 
     def is_started(self):
-        return self.started
+        return self.started or self.submitted
 
     def is_running(self):
-        if self.status in [WorkStatus.Running]:
+        if self.status in [WorkStatus.Running, WorkStatus.Transforming]:
             return True
         return False
 
-    def is_terminated(self):
+    def is_terminated(self, synchronize=True):
         """
         *** Function called by Transformer agent.
         """
@@ -1309,7 +1373,7 @@ class Work(Base):
             return True
         return False
 
-    def is_finished(self):
+    def is_finished(self, synchronize=True):
         """
         *** Function called by Transformer agent.
         """
@@ -1317,7 +1381,7 @@ class Work(Base):
             return True
         return False
 
-    def is_subfinished(self):
+    def is_subfinished(self, synchronize=True):
         """
         *** Function called by Transformer agent.
         """
@@ -1325,7 +1389,7 @@ class Work(Base):
             return True
         return False
 
-    def is_failed(self):
+    def is_failed(self, synchronize=True):
         """
         *** Function called by Transformer agent.
         """
@@ -1333,7 +1397,7 @@ class Work(Base):
             return True
         return False
 
-    def is_expired(self):
+    def is_expired(self, synchronize=True):
         """
         *** Function called by Transformer agent.
         """
@@ -1341,7 +1405,7 @@ class Work(Base):
             return True
         return False
 
-    def is_cancelled(self):
+    def is_cancelled(self, synchronize=True):
         """
         *** Function called by Transformer agent.
         """
@@ -1349,7 +1413,7 @@ class Work(Base):
             return True
         return False
 
-    def is_suspended(self):
+    def is_suspended(self, synchronize=True):
         """
         *** Function called by Transformer agent.
         """
@@ -1504,7 +1568,7 @@ class Work(Base):
     def get_other_output_collections(self):
         return [self.collections[k] for k in self._other_output_collections]
 
-    def get_input_collections(self):
+    def get_input_collections(self, poll_externel=False):
         """
         *** Function called by Transformer agent.
         """
@@ -1550,6 +1614,9 @@ class Work(Base):
             return []
         return []
 
+    def poll_external_collection(self, coll):
+        return coll
+
     def poll_internal_collection(self, coll):
         try:
             if coll.status in [CollectionStatus.Closed]:
@@ -1649,6 +1716,15 @@ class Work(Base):
     def set_has_new_inputs(self, yes=True):
         self.has_new_inputs = yes
 
+    def has_dependency(self):
+        return False
+
+    def get_parent_work_names(self):
+        return []
+
+    def get_parent_workload_ids(self):
+        return []
+
     def get_new_input_output_maps(self, mapped_input_output_maps={}):
         """
         *** Function called by Transformer agent.
@@ -1958,7 +2034,7 @@ class Work(Base):
         """
         raise exceptions.NotImplementedException
 
-    def abort_processing(self, processing):
+    def abort_processing_old(self, processing):
         """
         *** Function called by Carrier agent.
         """
@@ -1980,7 +2056,7 @@ class Work(Base):
             proc = processing['processing_metadata']['processing']
             proc.tosuspend = True
 
-    def resume_processing(self, processing):
+    def resume_processing_old(self, processing):
         """
         *** Function called by Carrier agent.
         """
@@ -2070,7 +2146,7 @@ class Work(Base):
             self.started = True
         self.logger.debug("syn_work_status(%s): work.status: %s" % (str(self.get_processing_ids()), str(self.status)))
 
-    def sync_work_data(self, status, substatus, work):
+    def sync_work_data(self, status, substatus, work, workload_id=None, output_data=None, processing=None):
         # self.status = work.status
         work.work_id = self.work_id
         work.transforming = self.transforming
@@ -2078,12 +2154,30 @@ class Work(Base):
         # clerk will update next_works while transformer doesn't.
         # synchronizing work metadata from transformer to clerk needs to keep it at first.
         next_works = self.next_works
-        self.metadata = work.metadata
+        # self.metadata = work.metadata
         self.next_works = next_works
 
         self.status_statistics = work.status_statistics
-        self.processings = work.processings
-        self.output_data = work.output_data
+        # self.processings = work.processings
+        if output_data:
+            self.output_data = output_data
+        else:
+            self.output_data = work.output_data
+
+        self.status = get_work_status_from_transform_processing_status(status)
+        self.substatus = get_work_status_from_transform_processing_status(substatus)
+        if workload_id:
+            self.workload_id = workload_id
+        if processing is not None:
+            # called by transformer to sync from processing
+            if processing.submitted_at:
+                self.submitted = True
+        else:
+            # called by clerk to syn from transform
+            self.submitted = work.submitted
+
+        if self.submitted:
+            self.started = True
 
         """
         self.status = WorkStatus(status.value)
@@ -2105,6 +2199,14 @@ class Work(Base):
         self.suspended_processings = work.suspended_processings
         """
 
+    def abort_processing(self, processing, log_prefix=''):
+        msg = "abort processing is not implemented"
+        self.logger.error(log_prefix + msg)
+
+    def resume_processing(self, processing, log_prefix=''):
+        msg = "resume processing is not implemented"
+        self.logger.error(log_prefix + msg)
+
     def add_proxy(self, proxy):
         self.proxy = proxy
 
