@@ -13,7 +13,6 @@ import datetime
 import logging
 import inspect
 import random
-import time
 import uuid
 
 
@@ -615,7 +614,8 @@ class WorkflowBase(Base):
             self._name = 'idds.workflow.' + datetime.datetime.utcnow().strftime("%Y_%m_%d_%H_%M_%S_%f") + str(random.randint(1, 1000))
 
         if workload_id is None:
-            workload_id = int(time.time())
+            # workload_id = int(time.time())
+            pass
         self.workload_id = workload_id
 
         self.logger = logger
@@ -636,6 +636,7 @@ class WorkflowBase(Base):
 
         self.first_initial = False
         self.new_to_run_works = []
+        self.submitting_works = []
         self.current_running_works = []
 
         self.num_subfinished_works = 0
@@ -667,6 +668,10 @@ class WorkflowBase(Base):
 
         self.global_parameters = {}
 
+        self.to_cancel = False
+
+        self.synchronized = False
+
         """
         self._running_data_names = []
         for name in ['internal_id', 'template_work_id', 'workload_id', 'work_sequence', 'terminated_works',
@@ -881,17 +886,47 @@ class WorkflowBase(Base):
     def set_global_parameters(self, value):
         self.global_parameters = value
 
+    @property
+    def sliced_global_parameters(self):
+        self._sliced_global_parameters = self.get_metadata_item('sliced_gp', {})
+        return self._sliced_global_parameters
+
+    @sliced_global_parameters.setter
+    def sliced_global_parameters(self, value):
+        self._sliced_global_parameters = value
+        gp_metadata = {}
+        if self._sliced_global_parameters:
+            for key in self._sliced_global_parameters:
+                if key.startswith("user_"):
+                    gp_metadata[key] = self._sliced_global_parameters[key]
+                else:
+                    self.logger.warn("Only parameters start with 'user_' can be set as global parameters. The parameter '%s' will be ignored." % (key))
+        self.add_metadata_item('sliced_gp', gp_metadata)
+
+    def set_sliced_global_parameters(self, source, name=None, index=0):
+        sliced_global_parameters = self.sliced_global_parameters
+        sliced_global_parameters[source] = {'name': name, 'index': index}
+        # to trigger the setter function
+        self.sliced_global_parameters = sliced_global_parameters
+
+    def sync_global_parameters(self, global_parameters, sliced_global_parameters=None):
+        gp = self.global_parameters
+        for key in global_parameters:
+            gp[key] = global_parameters[key]
+        self.global_parameters = gp
+
     def sync_global_parameters_from_work(self, work):
-        self.log_debug("work %s is_terminated, global_parameters: %s" % (work.get_internal_id(), str(self.global_parameters)))
-        if self.global_parameters:
-            for key in self.global_parameters:
-                status, value = work.get_global_parameter_from_output_data(key)
-                self.log_debug("work %s get_global_parameter_from_output_data(key: %s) results(%s:%s)" % (work.get_internal_id(), key, status, value))
-                if status:
-                    self.global_parameters[key] = value
-                elif hasattr(work, key):
-                    self.global_parameters[key] = getattr(work, key)
-        self.set_global_parameters(self.global_parameters)
+        self.log_debug("work %s (%s) is_terminated, global_parameters: %s" % (work.get_internal_id(), str(work), str(self.global_parameters)))
+        if isinstance(work, Work):
+            if self.global_parameters:
+                for key in self.global_parameters:
+                    status, value = work.get_global_parameter_from_output_data(key)
+                    self.log_debug("work %s get_global_parameter_from_output_data(key: %s) results(%s:%s)" % (work.get_internal_id(), key, status, value))
+                    if status:
+                        self.global_parameters[key] = value
+                    elif hasattr(work, key):
+                        self.global_parameters[key] = getattr(work, key)
+            self.set_global_parameters(self.global_parameters)
 
     @property
     def loop_condition(self):
@@ -928,6 +963,14 @@ class WorkflowBase(Base):
     def first_initial(self, value):
         self.add_metadata_item('first_initial', value)
 
+    @property
+    def to_start_works(self):
+        return self.get_metadata_item('to_start_works', [])
+
+    @to_start_works.setter
+    def to_start_works(self, value):
+        self.add_metadata_item('to_start_works', value)
+
     @property
     def new_to_run_works(self):
         return self.get_metadata_item('new_to_run_works', [])
@@ -936,6 +979,14 @@ class WorkflowBase(Base):
     def new_to_run_works(self, value):
         self.add_metadata_item('new_to_run_works', value)
 
+    @property
+    def submitting_works(self):
+        return self.get_metadata_item('submitting_works', [])
+
+    @submitting_works.setter
+    def submitting_works(self, value):
+        self.add_metadata_item('submitting_works', value)
+
     @property
     def current_running_works(self):
         return self.get_metadata_item('current_running_works', [])
@@ -1032,6 +1083,22 @@ class WorkflowBase(Base):
     def num_run(self, value):
         self.add_metadata_item('num_run', value)
 
+    @property
+    def parent_num_run(self):
+        return self.get_metadata_item('parent_num_run', 0)
+
+    @parent_num_run.setter
+    def parent_num_run(self, value):
+        self.add_metadata_item('parent_num_run', value)
+
+    @property
+    def to_cancel(self):
+        return self.get_metadata_item('to_cancel', False)
+
+    @to_cancel.setter
+    def to_cancel(self, value):
+        self.add_metadata_item('to_cancel', value)
+
     def load_metadata(self):
         self.load_works()
         self.load_work_conditions()
@@ -1083,15 +1150,30 @@ class WorkflowBase(Base):
     def get_works(self):
         return self.works
 
+    def get_combined_num_run(self):
+        if self.parent_num_run and len(str(self.parent_num_run)):
+            if self.num_run and len(str(self.num_run)):
+                return str(self.parent_num_run) + "_" + str(self.num_run)
+            else:
+                return str(self.parent_num_run)
+        else:
+            if self.num_run and len(str(self.num_run)):
+                return str(self.num_run)
+            else:
+                return ""
+
     def get_new_work_to_run(self, work_id, new_parameters=None):
         # 1. initialize works
         # template_id = work.get_template_id()
         work = self.works[work_id]
+        work.workload_id = None
+
         if isinstance(work, Workflow):
-            work.sync_works()
+            work.parent_num_run = self.get_combined_num_run()
+            work.sync_global_parameters(self.global_parameters, self.sliced_global_parameters)
+            work.sync_works(to_cancel=self.to_cancel)
 
             work.sequence_id = self.num_total_works
-            work.parent_num_run = self.num_run
 
             works = self.works
             self.works = works
@@ -1106,10 +1188,16 @@ class WorkflowBase(Base):
                 work.set_parameters(new_parameters)
             work.sequence_id = self.num_total_works
 
-            work.num_run = self.num_run
+            work.num_run = self.get_combined_num_run()
             work.initialize_work()
-            work.sync_global_parameters(self.global_parameters)
+            work.sync_global_parameters(self.global_parameters, self.sliced_global_parameters)
             work.renew_parameters_from_attributes()
+            if work.parent_workload_id is None and self.num_total_works > 0:
+                last_work_id = self.work_sequence[str(self.num_total_works - 1)]
+                last_work = self.works[last_work_id]
+                if isinstance(last_work, Work):
+                    work.parent_workload_id = last_work.workload_id
+                last_work.add_next_work(work.get_internal_id())
             works = self.works
             self.works = works
             # self.work_sequence.append(new_work.get_internal_id())
@@ -1127,7 +1215,7 @@ class WorkflowBase(Base):
         work.sequence_id = self.num_total_works
 
         work.initialize_work()
-        work.sync_global_parameters(self.global_parameters)
+        work.sync_global_parameters(self.global_parameters, self.sliced_global_parameters)
         work.renew_parameters_from_attributes()
         works = self.works
         self.works = works
@@ -1311,6 +1399,8 @@ class WorkflowBase(Base):
                 # parameters = self.get_destination_parameters(next_work.get_internal_id())
                 new_next_work = self.get_new_work_to_run(next_work.get_internal_id())
                 work.add_next_work(new_next_work.get_internal_id())
+                if isinstance(work, Work):
+                    new_next_work.parent_workload_id = work.workload_id
                 # cond.add_condition_work(new_next_work)   ####### TODO:
                 new_next_works.append(new_next_work)
             return new_next_works
@@ -1334,23 +1424,49 @@ class WorkflowBase(Base):
     def __str__(self):
         return str(json_dumps(self))
 
-    def get_new_works(self):
+    def get_new_works(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
 
         new works to be ready to start
         """
-        self.sync_works()
+        self.logger.info("%s get_new_works" % self.get_internal_id())
+        if self.to_cancel:
+            return []
+
+        if synchronize:
+            self.sync_works(to_cancel=self.to_cancel)
         works = []
+
+        if self.submitting_works:
+            # wait the work to be submitted
+            return works
+
+        if self.to_start_works:
+            to_start_works = self.to_start_works.copy()
+            init_works = self.init_works
+            starting_works = []
+            for work_id in to_start_works:
+                if not self.works[work_id].has_dependency():
+                    starting_works.append(work_id)
+                    self.get_new_work_to_run(work_id)
+                    if not init_works:
+                        init_works.append(work_id)
+                        self.init_works = init_works
+            for work_id in starting_works:
+                if work_id in self.to_start_works:
+                    self.to_start_works.remove(work_id)
+
         for k in self.new_to_run_works:
             if isinstance(self.works[k], Work):
                 self.works[k] = self.get_new_parameters_for_work(self.works[k])
                 works.append(self.works[k])
             if isinstance(self.works[k], Workflow):
-                works = works + self.works[k].get_new_works()
+                works = works + self.works[k].get_new_works(synchronize=False)
         for k in self.current_running_works:
             if isinstance(self.works[k], Workflow):
-                works = works + self.works[k].get_new_works()
+                works = works + self.works[k].get_new_works(synchronize=False)
+        self.logger.info("%s get_new_works done" % self.get_internal_id())
         return works
 
     def get_current_works(self):
@@ -1359,7 +1475,7 @@ class WorkflowBase(Base):
 
         Current running works
         """
-        self.sync_works()
+        self.sync_works(to_cancel=self.to_cancel)
         works = []
         for k in self.current_running_works:
             if isinstance(self.works[k], Work):
@@ -1368,20 +1484,23 @@ class WorkflowBase(Base):
                 works = works + self.works[k].get_current_works()
         return works
 
-    def get_all_works(self):
+    def get_all_works(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
 
         Current running works
         """
-        self.sync_works()
+        self.logger.info("%s get_all_works" % self.get_internal_id())
+        if synchronize:
+            self.sync_works(to_cancel=self.to_cancel)
 
         works = []
         for k in self.works:
             if isinstance(self.works[k], Work):
                 works.append(self.works[k])
             if isinstance(self.works[k], Workflow):
-                works = works + self.works[k].get_all_works()
+                works = works + self.works[k].get_all_works(synchronize=False)
+        self.logger.info("%s get_all_works done" % self.get_internal_id())
         return works
 
     def get_primary_initial_collection(self):
@@ -1492,33 +1611,45 @@ class WorkflowBase(Base):
                 tostart_works = list(self.get_works().keys())
                 tostart_works = [tostart_works[0]]
 
-            init_works = []
+            to_start_works = self.to_start_works
             for work_id in tostart_works:
-                self.get_new_work_to_run(work_id)
-                init_works.append(work_id)
-            self.init_works = init_works
+                to_start_works.append(work_id)
+            self.to_start_works = to_start_works
             self.log_debug("first initialized")
 
-    def sync_works(self):
-        self.log_debug("synchroning works")
+    def sync_works(self, to_cancel=False):
+        if to_cancel:
+            self.to_cancel = to_cancel
+        self.log_debug("%s synchroning works" % self.get_internal_id())
         self.first_initialize()
 
         self.refresh_works()
 
         for k in self.works:
             work = self.works[k]
-            self.log_debug("work %s is_terminated(%s:%s)" % (work.get_internal_id(), work.is_terminated(), work.get_status()))
+            self.log_debug("work %s is_terminated(%s:%s), is_submitted: %s, transforming: %s" % (work.get_internal_id(),
+                                                                                                 work.is_terminated(synchronize=False),
+                                                                                                 work.get_status(),
+                                                                                                 work.submitted,
+                                                                                                 work.transforming))
 
+        submitting_works = self.submitting_works
         for work in [self.works[k] for k in self.new_to_run_works]:
             if work.transforming:
                 self.new_to_run_works.remove(work.get_internal_id())
+                submitting_works.append(work.get_internal_id())
                 self.current_running_works.append(work.get_internal_id())
+        self.submitting_works = submitting_works
+        for work in [self.works[k] for k in self.submitting_works]:
+            self.log_info("Work %s is_submitted(%s)" % (work.get_internal_id(), work.submitted))
+            if work.submitted:
+                self.submitting_works.remove(work.get_internal_id())
 
         for work in [self.works[k] for k in self.current_running_works]:
             if isinstance(work, Workflow):
-                work.sync_works()
+                work.sync_works(to_cancel=self.to_cancel)
 
-            if work.is_terminated():
+            if work.is_terminated(synchronize=False):
                 self.log_debug("work %s is_terminated, sync_global_parameters_from_work" % (work.get_internal_id()))
                 self.set_source_parameters(work.get_internal_id())
                 self.sync_global_parameters_from_work(work)
@@ -1532,7 +1663,7 @@ class WorkflowBase(Base):
                                                                              json_dumps(cond, sort_keys=True, indent=4)))
                     self.enable_next_works(work, cond)
 
-            if work.is_terminated():
+            if work.is_terminated(synchronize=False):
                 self.log_info("Work %s is terminated(%s)" % (work.get_internal_id(), work.get_status()))
                 self.log_debug("Work conditions: %s" % json_dumps(self.work_conds, sort_keys=True, indent=4))
                 if work.get_internal_id() not in self.work_conds:
@@ -1548,36 +1679,39 @@ class WorkflowBase(Base):
                     self.terminated_works.append(work.get_internal_id())
                     self.current_running_works.remove(work.get_internal_id())
 
-                if work.is_finished():
+                if work.is_finished(synchronize=False):
                     self.num_finished_works += 1
-                elif work.is_subfinished():
+                elif work.is_subfinished(synchronize=False):
                     self.num_subfinished_works += 1
-                elif work.is_failed():
+                elif work.is_failed(synchronize=False):
                     self.num_failed_works += 1
-                elif work.is_expired():
+                elif work.is_expired(synchronize=False):
                     self.num_expired_works += 1
-                elif work.is_cancelled():
+                elif work.is_cancelled(synchronize=False):
                     self.num_cancelled_works += 1
-                elif work.is_suspended():
+                elif work.is_suspended(synchronize=False):
                     self.num_suspended_works += 1
 
             # if work.is_terminated():
             #    # if it's a loop workflow, to generate new loop
             #    if isinstance(work, Workflow):
             #        work.sync_works()
-        log_str = "num_total_works: %s" % self.num_total_works
+        log_str = "%s num_total_works: %s" % (self.get_internal_id(), self.num_total_works)
         log_str += ", num_finished_works: %s" % self.num_finished_works
         log_str += ", num_subfinished_works: %s" % self.num_subfinished_works
         log_str += ", num_failed_works: %s" % self.num_failed_works
         log_str += ", num_expired_works: %s" % self.num_expired_works
         log_str += ", num_cancelled_works: %s" % self.num_cancelled_works
         log_str += ", num_suspended_works: %s" % self.num_suspended_works
+        log_str += ", new_to_run_works: %s" % len(self.new_to_run_works)
+        log_str += ", current_running_works: %s" % len(self.current_running_works)
         self.log_debug(log_str)
 
         self.refresh_works()
-        self.log_debug("synchronized works")
+        self.log_debug("%s synchronized works" % self.get_internal_id())
 
     def resume_works(self):
+        self.to_cancel = False
         self.num_subfinished_works = 0
         self.num_finished_works = 0
         self.num_failed_works = 0
@@ -1712,32 +1846,37 @@ class WorkflowBase(Base):
         """
         return [self]
 
-    def is_terminated(self):
+    def is_terminated(self, synchronize=True, new=False):
         """
         *** Function called by Marshaller agent.
         """
-        self.sync_works()
-        if len(self.new_to_run_works) == 0 and len(self.current_running_works) == 0:
-            return True
+        if synchronize:
+            self.sync_works(to_cancel=self.to_cancel)
+        if new:
+            if (self.to_cancel) or (len(self.new_to_run_works) == 0 and len(self.current_running_works) == 0 and self.num_total_works > 0):
+                return True
+        else:
+            if (self.to_cancel or len(self.new_to_run_works) == 0) and len(self.current_running_works) == 0:
+                return True
         return False
 
-    def is_finished(self):
+    def is_finished(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
         """
-        return self.is_terminated() and self.num_finished_works == self.num_total_works
+        return self.is_terminated(synchronize=synchronize) and self.num_finished_works == self.num_total_works
 
-    def is_subfinished(self):
+    def is_subfinished(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
         """
-        return self.is_terminated() and (self.num_finished_works + self.num_subfinished_works > 0 and self.num_finished_works + self.num_subfinished_works <= self.num_total_works)
+        return self.is_terminated(synchronize=synchronize) and (self.num_finished_works + self.num_subfinished_works > 0 and self.num_finished_works + self.num_subfinished_works <= self.num_total_works)
 
-    def is_failed(self):
+    def is_failed(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
         """
-        return self.is_terminated() and (self.num_failed_works > 0) and (self.num_cancelled_works == 0) and (self.num_suspended_works == 0) and (self.num_expired_works == 0)
+        return self.is_terminated(synchronize=synchronize) and (self.num_failed_works > 0) and (self.num_cancelled_works == 0) and (self.num_suspended_works == 0) and (self.num_expired_works == 0)
 
     def is_to_expire(self, expired_at=None, pending_time=None, request_id=None):
         if self.expired:
@@ -1771,24 +1910,24 @@ class WorkflowBase(Base):
 
         return False
 
-    def is_expired(self):
+    def is_expired(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
         """
         # return self.is_terminated() and (self.num_expired_works > 0)
-        return self.is_terminated() and self.expired
+        return self.is_terminated(synchronize=synchronize) and self.expired
 
-    def is_cancelled(self):
+    def is_cancelled(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
         """
-        return self.is_terminated() and (self.num_cancelled_works > 0)
+        return self.is_terminated(synchronize=synchronize) and (self.num_cancelled_works > 0)
 
-    def is_suspended(self):
+    def is_suspended(self, synchronize=True):
         """
         *** Function called by Marshaller agent.
         """
-        return self.is_terminated() and (self.num_suspended_works > 0)
+        return self.is_terminated(synchronize=synchronize) and (self.num_suspended_works > 0)
 
     def get_terminated_msg(self):
         """
@@ -1800,17 +1939,17 @@ class WorkflowBase(Base):
 
     def get_status(self):
         if self.is_terminated():
-            if self.is_finished():
+            if self.is_finished(synchronize=False):
                 return WorkStatus.Finished
-            elif self.is_subfinished():
+            elif self.is_subfinished(synchronize=False):
                 return WorkStatus.SubFinished
-            elif self.is_failed():
+            elif self.is_failed(synchronize=False):
                 return WorkStatus.Failed
-            elif self.is_expired():
+            elif self.is_expired(synchronize=False):
                 return WorkStatus.Expired
-            elif self.is_cancelled():
+            elif self.is_cancelled(synchronize=False):
                 return WorkStatus.Cancelled
-            elif self.is_suspended():
+            elif self.is_suspended(synchronize=False):
                 return WorkStatus.Suspended
         return WorkStatus.Transforming
 
@@ -1834,6 +1973,8 @@ class Workflow(Base):
             self.setup_logger()
 
         self.template = WorkflowBase(name=name, workload_id=workload_id, lifetime=lifetime, pending_time=pending_time, logger=logger)
+
+        # parent_num_run is string.
         self.parent_num_run = None
         self._num_run = 0
         self.runs = {}
@@ -1895,8 +2036,10 @@ class Workflow(Base):
             parameter_links = run_metadata['parameter_links']
             self.template.set_parameter_links_metadata(parameter_links)
         for run_id in runs:
+            self.template.parent_num_run = self.parent_num_run
             self.runs[run_id] = self.template.copy()
             self.runs[run_id].metadata = runs[run_id]
+            self.runs[run_id].parent_num_run = self.parent_num_run
         # self.add_metadata_item('runs', )
 
     @property
@@ -1960,18 +2103,28 @@ class Workflow(Base):
     def lifetime(self, value):
         self.template.lifetime = value
 
+    @property
+    def to_cancel(self):
+        return self.template.to_cancel
+
+    @to_cancel.setter
+    def to_cancel(self, value):
+        if self.runs:
+            self.runs[str(self.num_run)].to_cancel = value
+        self.template.to_cancel = value
+
     @property
     def num_run(self):
-        if self.parent_num_run:
-            return self.parent_num_run * 100 + self._num_run
         return self._num_run
 
     @num_run.setter
     def num_run(self, value):
+        self._num_run = value
+
+    def get_combined_num_run(self):
         if self.parent_num_run:
-            self._num_run = value - self.parent_num_run * 100
-        else:
-            self._num_run = value
+            return str(self.parent_num_run) + "_" + str(self.num_run)
+        return str(self.num_run)
 
     @property
     def transforming(self):
@@ -1984,13 +2137,29 @@ class Workflow(Base):
         if self._num_run < 1:
             self._num_run = 1
         if str(self.num_run) not in self.runs:
+            self.template.parent_num_run = self.parent_num_run
             self.runs[str(self.num_run)] = self.template.copy()
             if self.runs[str(self.num_run)].has_loop_condition():
                 self.runs[str(self.num_run)].num_run = self.num_run
+                self.runs[str(self.num_run)].parent_num_run = self.parent_num_run
+
+            # self.runs[str(self.num_run)].parent_num_run = self.get_combined_num_run()
+            if self.runs[str(self.num_run)].has_loop_condition():
                 if self._num_run > 1:
                     p_metadata = self.runs[str(self.num_run - 1)].get_metadata_item('parameter_links')
                     self.runs[str(self.num_run)].add_metadata_item('parameter_links', p_metadata)
 
+    @property
+    def submitted(self):
+        return self.transforming
+
+    @submitted.setter
+    def submitted(self, value):
+        pass
+
+    def has_dependency(self):
+        return False
+
     def set_workload_id(self, workload_id):
         if self.runs:
             self.runs[str(self.num_run)].workload_id = workload_id
@@ -2032,30 +2201,46 @@ class Workflow(Base):
     def set_global_parameters(self, value):
         self.template.set_global_parameters(value)
 
+    def set_sliced_global_parameters(self, source, name=None, index=0):
+        self.template.set_sliced_global_parameters(source, name=name, index=index)
+
     def sync_global_parameters_from_work(self, work):
         if self.runs:
             return self.runs[str(self.num_run)].sync_global_parameters_from_work(work)
         return self.template.sync_global_parameters_from_work(work)
 
-    def get_new_works(self):
-        self.log_debug("synchronizing works")
-        self.sync_works()
-        self.log_debug("synchronized works")
+    def sync_global_parameters(self, global_parameters, sliced_global_parameters=None):
         if self.runs:
-            return self.runs[str(self.num_run)].get_new_works()
-        return []
+            return self.runs[str(self.num_run)].sync_global_parameters(global_parameters, sliced_global_parameters)
+        return self.template.sync_global_parameters(global_parameters, sliced_global_parameters)
+
+    def get_new_works(self, synchronize=True):
+        self.logger.info("%s get_new_works" % self.get_internal_id())
+        self.log_debug("synchronizing works")
+        if synchronize:
+            self.sync_works(to_cancel=self.to_cancel)
+        self.log_debug("synchronized works")
+        works = []
+        if self.runs:
+            works = self.runs[str(self.num_run)].get_new_works(synchronize=False)
+        self.logger.info("%s get_new_works done" % self.get_internal_id())
+        return works
 
     def get_current_works(self):
-        self.sync_works()
+        self.sync_works(to_cancel=self.to_cancel)
         if self.runs:
             return self.runs[str(self.num_run)].get_current_works()
         return []
 
-    def get_all_works(self):
-        self.sync_works()
+    def get_all_works(self, synchronize=True):
+        self.logger.info("%s get_all_works" % self.get_internal_id())
+        if synchronize:
+            self.sync_works(to_cancel=self.to_cancel)
+        works = []
         if self.runs:
-            return self.runs[str(self.num_run)].get_all_works()
-        return []
+            works = self.runs[str(self.num_run)].get_all_works(synchronize=False)
+        self.logger.info("%s get_all_works done" % self.get_internal_id())
+        return works
 
     def get_primary_initial_collection(self):
         if self.runs:
@@ -2065,6 +2250,7 @@ class Workflow(Base):
     def resume_works(self):
         if self.runs:
             self.runs[str(self.num_run)].resume_works()
+        self.template.to_cancel = False
 
     def clean_works(self):
         # if self.runs:
@@ -2078,41 +2264,41 @@ class Workflow(Base):
             return self.runs[str(self.num_run)].is_to_expire(expired_at=expired_at, pending_time=pending_time, request_id=request_id)
         return False
 
-    def is_terminated(self):
+    def is_terminated(self, synchronize=True):
         if self.runs:
-            if self.runs[str(self.num_run)].is_terminated():
+            if self.runs[str(self.num_run)].is_terminated(synchronize=synchronize):
                 if not self.runs[str(self.num_run)].has_loop_condition() or not self.runs[str(self.num_run)].get_loop_condition_status():
                     return True
         return False
 
-    def is_finished(self):
-        if self.is_terminated():
-            return self.runs[str(self.num_run)].is_finished()
+    def is_finished(self, synchronize=True):
+        if self.is_terminated(synchronize=synchronize):
+            return self.runs[str(self.num_run)].is_finished(synchronize=False)
         return False
 
-    def is_subfinished(self):
-        if self.is_terminated():
-            return self.runs[str(self.num_run)].is_subfinished()
+    def is_subfinished(self, synchronize=True):
+        if self.is_terminated(synchronize=synchronize):
+            return self.runs[str(self.num_run)].is_subfinished(synchronize=False)
         return False
 
-    def is_failed(self):
-        if self.is_terminated():
-            return self.runs[str(self.num_run)].is_failed()
+    def is_failed(self, synchronize=True):
+        if self.is_terminated(synchronize=synchronize):
+            return self.runs[str(self.num_run)].is_failed(synchronize=False)
         return False
 
-    def is_expired(self):
-        if self.is_terminated():
-            return self.runs[str(self.num_run)].is_expired()
+    def is_expired(self, synchronize=True):
+        if self.is_terminated(synchronize=synchronize):
+            return self.runs[str(self.num_run)].is_expired(synchronize=False)
         return False
 
-    def is_cancelled(self):
-        if self.is_terminated():
-            return self.runs[str(self.num_run)].is_cancelled()
+    def is_cancelled(self, synchronize=True):
+        if self.is_terminated(synchronize=synchronize):
+            return self.runs[str(self.num_run)].is_cancelled(synchronize=False)
         return False
 
-    def is_suspended(self):
-        if self.is_terminated():
-            return self.runs[str(self.num_run)].is_suspended()
+    def is_suspended(self, synchronize=True):
+        if self.is_terminated(synchronize=synchronize):
+            return self.runs[str(self.num_run)].is_suspended(synchronize=False)
         return False
 
     def get_terminated_msg(self):
@@ -2147,34 +2333,50 @@ class Workflow(Base):
         if self.runs:
             self.runs[str(self.num_run)].refresh_works()
 
-    def sync_works(self):
+    def sync_works(self, to_cancel=False):
+        if to_cancel:
+            self.to_cancel = to_cancel
+
+        self.refresh_works()
         # position is end.
-        if self._num_run < 1:
-            self._num_run = 1
+        if self.num_run < 1:
+            self.num_run = 1
         if str(self.num_run) not in self.runs:
+            self.template.parent_num_run = self.parent_num_run
             self.runs[str(self.num_run)] = self.template.copy()
             if self.runs[str(self.num_run)].has_loop_condition():
-                self.runs[str(self.num_run)].num_run = self._num_run
-                if self._num_run > 1:
-                    p_metadata = self.runs[str(self.num_run - 1)].get_metadata_item('parameter_links')
-                    self.runs[str(self.num_run)].add_metadata_item('parameter_links', p_metadata)
+                self.runs[str(self.num_run)].num_run = self.num_run
 
-        self.runs[str(self.num_run)].sync_works()
-
-        if self.runs[str(self.num_run)].is_terminated():
+            # self.runs[str(self.num_run)].parent_num_run = self.get_combined_num_run()
             if self.runs[str(self.num_run)].has_loop_condition():
-                if self.runs[str(self.num_run)].get_loop_condition_status():
-                    self.logger.info("num_run %s get_loop_condition_status %s, start next run" % (self.num_run, self.runs[str(self.num_run)].get_loop_condition_status()))
-                    self._num_run += 1
-                    self.runs[str(self.num_run)] = self.template.copy()
-
-                    self.runs[str(self.num_run)].num_run = self._num_run
+                if self.num_run > 1:
                     p_metadata = self.runs[str(self.num_run - 1)].get_metadata_item('parameter_links')
                     self.runs[str(self.num_run)].add_metadata_item('parameter_links', p_metadata)
 
-                    self.runs[str(self.num_run)].global_parameters = self.runs[str(self.num_run - 1)].global_parameters
-                else:
-                    self.logger.info("num_run %s get_loop_condition_status %s, terminated loop" % (self.num_run, self.runs[str(self.num_run)].get_loop_condition_status()))
+        self.runs[str(self.num_run)].sync_works(to_cancel=to_cancel)
+
+        if self.runs[str(self.num_run)].is_terminated(synchronize=False, new=True):
+            self.logger.info("%s num_run %s is_terminated" % (self.get_internal_id(), self.num_run))
+            if to_cancel:
+                self.logger.info("num_run %s, to cancel" % self.num_run)
+            else:
+                if self.runs[str(self.num_run)].has_loop_condition():
+                    if self.runs[str(self.num_run)].get_loop_condition_status():
+                        self.logger.info("%s num_run %s get_loop_condition_status %s, start next run" % (self.get_internal_id(), self.num_run, self.runs[str(self.num_run)].get_loop_condition_status()))
+                        self._num_run += 1
+                        self.template.parent_num_run = self.parent_num_run
+                        self.runs[str(self.num_run)] = self.template.copy()
+
+                        if self.runs[str(self.num_run)].has_loop_condition():
+                            self.runs[str(self.num_run)].num_run = self.num_run
+
+                        p_metadata = self.runs[str(self.num_run - 1)].get_metadata_item('parameter_links')
+                        self.runs[str(self.num_run)].add_metadata_item('parameter_links', p_metadata)
+
+                        self.runs[str(self.num_run)].global_parameters = self.runs[str(self.num_run - 1)].global_parameters
+                    else:
+                        self.logger.info("%s num_run %s get_loop_condition_status %s, terminated loop" % (self.get_internal_id(), self.num_run, self.runs[str(self.num_run)].get_loop_condition_status()))
+        self.refresh_works()
 
     def get_relation_map(self):
         if not self.runs:
